% Vertically mounted three-springs-in-series are attched to a 0.2kg mass
% and sellotaped to a card diary to increase air resistance.
% Position of the mass is determined via a PASCO ultrasonic transducer,
% and data is logged at 20Hz using the Capstone system. Capstone
% automatically determines velocity and acceleration via some form of
% numerical differentiation.
function damped_mass_on_spring_analysis
fsize = 14;  %Fontsize for graphs
tstart = 0;  tend = 60*3;  %Time interval

%Load data
[num,txt,raw] = xlsread( 'Damped mass on spring.xlsx');

%Ignore first few data points (oscillation hasn't started yet)
num = num(50:end,:);

%Determine position, velocity, acceleration, time in SI units
t = num(:,4); t = t - t(1);
x = num(:,1); v = num(:,2); a = num(:,3);

%Determine equilibrium height from the average position, and shift x
%such that equilibrium displacement is zero.
x = x - mean(x);

%Restricted data set (i.e. where effect of damping can be ignored)
aa = a(100:200); tt = t(100:200); xx = x(100:200);

%Determine line of best fit between a and x to determine period
[m,c,yfit,r] = lfit(xx,aa); T = 2*pi/sqrt(-m);

%Find peaks and plot these to determine exponential envelope
[tp,xp] = peakfindergeneral(t,x); N = length(xp);

%Linearize exponential envelope. Choose which set of data you want to use
%for this computation. NOTE THE ENVELOPE IS NOT EXPONENTIAL IN THE SAME WAY
%DURING THE WHOLE DECAY, SINCE THE DRAG FORCE WON'T BE LINEAR WITH SPEED.
i = find(xp>0); i = i(1:50);
Y = log( xp(i) ); X = tp(i);

%Determine exponential envelope in the form x = (+/-) A*exp(-k*t)
[m,c,YFIT,r] = lfit( X,Y);
k = -m; A = exp(c);
xpeak_plus = A*exp(-k*tp); xpeak_minus = -A*exp(-k*tp);

%Plot position vs time, and overlay exponential envelope
plot(t,x,'r-'); xlim( [tstart,tend] );
hold on; plot( tp, xpeak_plus ,'b--' );
plot( tp, xpeak_minus,'b--' );
set(gca,'fontsize',fsize); xlabel('time /s');
ylabel('position /m'); title('Position vs time of damped mass-spring system');
grid on; print( gcf, 'x vs t.png','-dpng','-r300'); clf;

%Plot just peaks
plot(tp,xp,'r-'); xlim( [tstart,tend] ); hold on;
plot( tp, xpeak_plus ,'b--' );
set(gca,'fontsize',fsize); xlabel('time /s');
ylabel('position /m'); title('Position vs time of damped mass-spring system');
grid on; print( gcf, 'xp vs t.png','-dpng','-r300'); clf;

%Plot velocity vs time
plot(t,v,'g-'); xlim( [tstart,tend] );
set(gca,'fontsize',fsize); xlabel('time /s'); ylabel('velocity /ms^{-1}');
title('Velocity vs time of damped mass-spring system');grid on;
print( gcf, 'v vs t.png','-dpng','-r300'); clf;

%Plot acceleration vs time
plot(t,a,'b-'); xlim( [tstart,tend] );
set(gca,'fontsize',fsize); xlabel('time /s');
ylabel('acceleration /ms^{-2}');
title('Acceleration vs time of damped mass-spring system');
grid on; print( gcf, 'a vs t.png','-dpng','-r300'); clf;

%Plot velocity vs position
plot(x,v,'b.-'); set(gca,'fontsize',fsize);
xlabel('position /m'); ylabel('velocity /ms^{-1}');
title('Velocity vs position of damped mass-spring system');
grid on; print( gcf, 'v vs x.png','-dpng','-r300'); clf;

%Plot acceleration vs position
plot(x,a,'b.'); hold on; plot(xx,aa,'r.'); plot(xx,yfit,'k-');
set(gca,'fontsize',fsize); xlabel('position /m');
ylabel('acceleration /ms^{-2}');
title({'Acceleration vs position of damped mass-spring system',...
    ['Period = ',num2str(T,3),'s']});
grid on; print( gcf, 'a vs x.png','-dpng','-r300');
close(gcf);

%%

%Linear regression function (y = mx + c)
function [m,c,yfit,r] = lfit(x,y)
cov_xy = mean(x.*y) - mean(x)*mean(y);
Vx = mean(x.^2) - mean(x)^2;
Vy = mean(y.^2) - mean(y)^2;
m = cov_xy/Vx;
c = mean(y) - m*mean(x);
yfit = m*x + c;
r = cov_xy/sqrt( Vx*Vy );

%%

%Peak finder function
function [xp,yp] = peakfindergeneral(x,y)
xp = []; yp = [];
for n=2:length(x)-1
    if ( y(n) > y(n-1) ) && ( y(n) > y(n+1) )
        xp = [xp,x(n)]; yp = [yp,y(n)];
    end   
end

%End of code