%pendulum
% Plots the time series and phase space (Poincare) diagram for a simple
% pendulum swinging on a frictionless, massless rod.
%
% LAST UPDATED by Andy French December 2021.

function pendulum

%Length of pendulum L in metres
L = 1.23;

%Strength of gravity g (N/kg)
g = 9.81;

%Effect of air resistance. Dimensionless number a = g*k/(4*pi^2 * m ).
%Air resistance force is k * L^2 * (theta_dot)^2 (N)
a = 1;

%Number of 'exact' pendulum periods
tormax = 10;

%Timestep for numeric computation
dtor = 0.01;

%Fontsize
fsize = 18;

%

%Compute SHM period /s
P = 2*pi*sqrt(L/g);

%Compare trajectories:
% (i) Small initial, amplitude SHM
% (ii) Verlet with no air resistance
% (iii) Verlet with air resistance
theta0 = pi/12;
[theta1,tor,thetadot1] = ptraj_exact( dtor, tormax, theta0 );
theta0 = pi/6;
[theta2,tor,thetadot2] = ptraj( dtor, tormax, theta0, 0 );
[theta3,tor,thetadot3] = ptraj( dtor, tormax, theta0, a );

%Plot time series
figure('name','pendulum time series','color',[1 1 1])
plot( tor, theta1, tor, theta2, tor, theta3 );
legend({'Exact SHM','Verlet a=0',['Verlet a=',num2str(a)]},'fontsize',fsize);
xlabel('Number of periods','fontsize',fsize);
ylabel('\theta /radians','fontsize',fsize);
title(['L=',num2str(L),'m, g=',num2str(g),'Nkg^{-1}, P=',num2str(P),'s '],'fontsize',fsize);
grid on; set(gca,'fontsize',fsize)
print( gcf,'pendulum time series.png','-r300','-dpng' ); close(gcf)

%Plot phase space (Poincare) diagram)
figure('name','pendulum phase space','color',[1 1 1])
plot( theta1, thetadot1/(2*pi), theta2, thetadot2/(2*pi), theta3, thetadot3/(2*pi) );
hold on
plot( theta1(1), thetadot1(1)/(2*pi), 'o', theta2(1), thetadot2(1)/(2*pi),'o',theta3(1), thetadot3(1)/(2*pi), 'o' );
plot( theta1(end), thetadot1(end)/(2*pi), '*', theta2(end), thetadot2(end)/(2*pi),'*',theta3(end), thetadot3(end)/(2*pi), '*' );
legend({'Exact SHM','Verlet a=0',['Verlet a=',num2str(a)]},'fontsize',fsize);
xlabel('\theta /radians','fontsize',fsize);
ylabel('P*\thetadot /2\pi','fontsize',fsize);
title(['L=',num2str(L),'m, g=',num2str(g),'Nkg^{-1}, P=',num2str(P),'s '],'fontsize',fsize);
grid on; axis tight; axis equal; set(gca,'fontsize',fsize)
print( gcf,'pendulum phase space.png','-r300','-dpng' )
close(gcf)

%%

%Compute pendulum trajectory assuming small angles and no friction
function [theta,tor,thetadot] = ptraj_exact( dtor, tormax, theta0 )
tor = 0:dtor:tormax;
theta = theta0*cos(2*pi*tor);
thetadot = -2*pi*theta0*sin(2*pi*tor);

%%

%Compute pendulum trajectory (including air resistance and taking into
%account large angles) using the Verlet method
function [theta,tor,thetadot] = ptraj( dtor, tormax, theta0, a )
thetadot = 0;
theta = theta0;
tor =  0:dtor:tormax;
N = numel(tor);
for n=1:N-1
    A = -4*(pi^2)*sin(theta(n)) - a*thetadot(n)*abs(thetadot(n));
    theta(n+1) = theta(n) + thetadot(n)*dtor + 0.5*A*dtor^2;
    AA = -4*(pi^2)*sin(theta(n+1)) - a*thetadot(n)*abs(thetadot(n));
    thetadot(n+1) = thetadot(n) + 0.5*dtor*( A + AA );
end

%End of code